@@ -167,13 +167,14 @@ def all_jupyter_notebook_to_py_files(Gs):
     created_files = []
     for G in Gs:
         for s in G.nodes():
-            cmd = s.cmd.split()
-            if cmd[0] == 'python':
-                for c in cmd:
-                    if c.endswith('.py'):
-                        if not os.path.exists(c) and os.path.exists(c[:-3]+'.ipynb'):
-                            created_files.append(jupyter_notebook_to_py_file(c[:-3]+'.ipynb'))
-                            print(bcolors.BOLD+'    Convert ' + c[:-3]+'.ipynb' +  ' to ' + c + ' file.' + bcolors.ENDC)
+            if s.cmd is not None:
+                cmd = s.cmd.split()
+                if cmd[0] == 'python':
+                    for c in cmd:
+                        if c.endswith('.py'):
+                            if not os.path.exists(c) and os.path.exists(c[:-3]+'.ipynb'):
+                                created_files.append(jupyter_notebook_to_py_file(c[:-3]+'.ipynb'))
+                                print(bcolors.BOLD+'    Convert ' + c[:-3]+'.ipynb' +  ' to ' + c + ' file.' + bcolors.ENDC)
     return created_files
 
 def jupyter_notebook_to_py_file(path_to_ipynb):
@@ -230,80 +231,81 @@ def exec_branch(dvc_files, branch_name, project_dir, no_exec, num_of_repeats, li
     with open('cc_execution_file.red.yml',"w") as f:
         print("batches:", file=f)
         for i in range(len(dvc_files)):
-            dvcfiles_to_execute = str(dvc_files[i])[1:-1].replace("'", "").replace('"', '').replace(' ', '')
-            # print("batches:", file=f)
-            print("  - inputs:", file=f)
-            print("      git_authentication_json:", file=f)
-            print("        class: File", file=f)
-            print("        connector:", file=f)
-            print("          access: {username: '{{" + git_path.replace('.', '_').replace('-',
-                                                                                          '_') + "_username}}', password: '{{" + git_path.replace(
-                '.', '_').replace('-', '_') + "_password}}', email: '{{" + git_path.replace('.', '_').replace('-',
-                                                                                                              '_') + "_email}}'}",
-                  file=f)
-            print("          command: dvc-cc-connector", file=f)
-            print("      git_path_to_working_repository: \"" + git_path + "\"", file=f)
-            print("      git_working_repository_owner: \"" + git_owner + "\"", file=f)
-            print("      git_working_repository_name: \"" + git_name + "\"", file=f)
-            print("      git_name_of_branch: \"" + branch_name + "\"", file=f)
-            print("      dvc_authentication_json:", file=f)
-            print("        class: File", file=f)
-            print("        connector:", file=f)
-            print("          access: {username: '{{" + dvc_server.replace('.', '_').replace('-',
-                                                                                            '_') + "_username}}', password: '{{" + dvc_server.replace(
-                '.', '_').replace('-', '_') + "_password}}'}", file=f)
-            print("          command: dvc-cc-connector", file=f)
-            print("      dvc_servername: \"" + dvc_server + "\"", file=f)
-            print("      dvc_path_to_working_repository: \"" + dvc_path + "\"", file=f)
+            for j in range(num_of_repeats):
+                dvcfiles_to_execute = str(dvc_files[i])[1:-1].replace("'", "").replace('"', '').replace(' ', '')
+                # print("batches:", file=f)
+                print("  - inputs:", file=f)
+                print("      git_authentication_json:", file=f)
+                print("        class: File", file=f)
+                print("        connector:", file=f)
+                print("          access: {username: '{{" + git_path.replace('.', '_').replace('-',
+                                                                                              '_') + "_username}}', password: '{{" + git_path.replace(
+                    '.', '_').replace('-', '_') + "_password}}', email: '{{" + git_path.replace('.', '_').replace('-',
+                                                                                                                  '_') + "_email}}'}",
+                      file=f)
+                print("          command: dvc-cc-connector", file=f)
+                print("      git_path_to_working_repository: \"" + git_path + "\"", file=f)
+                print("      git_working_repository_owner: \"" + git_owner + "\"", file=f)
+                print("      git_working_repository_name: \"" + git_name + "\"", file=f)
+                print("      git_name_of_branch: \"" + branch_name + "\"", file=f)
+                print("      dvc_authentication_json:", file=f)
+                print("        class: File", file=f)
+                print("        connector:", file=f)
+                print("          access: {username: '{{" + dvc_server.replace('.', '_').replace('-',
+                                                                                                '_') + "_username}}', password: '{{" + dvc_server.replace(
+                    '.', '_').replace('-', '_') + "_password}}'}", file=f)
+                print("          command: dvc-cc-connector", file=f)
+                print("      dvc_servername: \"" + dvc_server + "\"", file=f)
+                print("      dvc_path_to_working_repository: \"" + dvc_path + "\"", file=f)
 
-            if sshfs_data is not None:
-                print("      sshfs_input_server_settings:", file=f)
-                for i in range(len(sshfs_data.keys())):
-                    sshfs_dest_rel = list(sshfs_data.keys())[i]
-                    sshfs_username = sshfs_data[sshfs_dest_rel]["username"]
-                    sshfs_server = sshfs_data[sshfs_dest_rel]["server"]
-                    sshfs_path = sshfs_data[sshfs_dest_rel]["remote_path"]
-                    sshfs_password = '{{' + str(sshfs_server).replace('.', '_').replace('-', '_') + '_password}}'
+                if sshfs_data is not None:
+                    print("      sshfs_input_server_settings:", file=f)
+                    for i in range(len(sshfs_data.keys())):
+                        sshfs_dest_rel = list(sshfs_data.keys())[i]
+                        sshfs_username = sshfs_data[sshfs_dest_rel]["username"]
+                        sshfs_server = sshfs_data[sshfs_dest_rel]["server"]
+                        sshfs_path = sshfs_data[sshfs_dest_rel]["remote_path"]
+                        sshfs_password = '{{' + str(sshfs_server).replace('.', '_').replace('-', '_') + '_password}}'
 
-                    print("        - class: Directory", file=f)
-                    print("          connector:", file=f)
-                    print("              command: \"red-connector-ssh\"", file=f)
-                    print("              mount: true", file=f)
-                    print("              access:", file=f)
-                    print("                host: '" + sshfs_server + "'", file=f)
-                    print("                port: 22", file=f)
-                    print("                auth:", file=f)
-                    print("                  username: '" + sshfs_username + "'", file=f)
-                    print("                  password: '" + sshfs_password + "'", file=f)
-                    print("                dirPath: '" + sshfs_path + "'", file=f)
+                        print("        - class: Directory", file=f)
+                        print("          connector:", file=f)
+                        print("              command: \"red-connector-ssh\"", file=f)
+                        print("              mount: true", file=f)
+                        print("              access:", file=f)
+                        print("                host: '" + sshfs_server + "'", file=f)
+                        print("                port: 22", file=f)
+                        print("                auth:", file=f)
+                        print("                  username: '" + sshfs_username + "'", file=f)
+                        print("                  password: '" + sshfs_password + "'", file=f)
+                        print("                dirPath: '" + sshfs_path + "'", file=f)
 
-                print("      sshfs_input_dest_rel_paths:", file=f)
-                for i in range(len(sshfs_data.keys())):
-                    sshfs_dest_rel = list(sshfs_data.keys())[i]
-                    print("        - '"+sshfs_dest_rel+"'", file=f)
+                    print("      sshfs_input_dest_rel_paths:", file=f)
+                    for i in range(len(sshfs_data.keys())):
+                        sshfs_dest_rel = list(sshfs_data.keys())[i]
+                        print("        - '"+sshfs_dest_rel+"'", file=f)
 
-            print("      dvc_remote_directory_sshfs:", file=f)
-            print("        class: Directory", file=f)
-            print("        connector:", file=f)
-            print("            command: \"red-connector-ssh\"", file=f)
-            print("            mount: true", file=f)
-            print("            access:", file=f)
-            print("              host: '" + dvc_server + "'", file=f)
-            print("              port: 22", file=f)
-            print("              auth:", file=f)
-            print("                username: '" + "{{" + dvc_server.replace('.', '_').replace('-',
-                                                                                              '_') + "_username}}'",
-                  file=f)
-            print("                password: '" + "{{" + dvc_server.replace(
-                '.', '_').replace('-', '_') + "_password}}" + "'", file=f)
-            print("              writable: True", file=f)
-            print("              dirPath: '" + dvc_path + "'", file=f)
+                print("      dvc_remote_directory_sshfs:", file=f)
+                print("        class: Directory", file=f)
+                print("        connector:", file=f)
+                print("            command: \"red-connector-ssh\"", file=f)
+                print("            mount: true", file=f)
+                print("            access:", file=f)
+                print("              host: '" + dvc_server + "'", file=f)
+                print("              port: 22", file=f)
+                print("              auth:", file=f)
+                print("                username: '" + "{{" + dvc_server.replace('.', '_').replace('-',
+                                                                                                  '_') + "_username}}'",
+                      file=f)
+                print("                password: '" + "{{" + dvc_server.replace(
+                    '.', '_').replace('-', '_') + "_password}}" + "'", file=f)
+                print("              writable: True", file=f)
+                print("              dirPath: '" + dvc_path + "'", file=f)
 
-            print("      dvc_file_to_execute: '" + dvcfiles_to_execute.replace('\\\\', '/') + "'", file=f)
-            if live_output_files is not None:
-                print("      live_output_files: '" + live_output_files + "'", file=f)
-                print("      live_output_update_frequence: " + str(live_output_update_frequence), file=f)
-            print("    outputs: {}", file=f)
+                print("      dvc_file_to_execute: '" + dvcfiles_to_execute.replace('\\\\', '/') + "'", file=f)
+                if live_output_files is not None:
+                    print("      live_output_files: '" + live_output_files + "'", file=f)
+                    print("      live_output_update_frequence: " + str(live_output_update_frequence), file=f)
+                print("    outputs: {}", file=f)
         with open('.dvc_cc/cc_config.yml',"r") as r:
             print(r.read(), file=f)
 
@@ -313,14 +315,20 @@ def exec_branch(dvc_files, branch_name, project_dir, no_exec, num_of_repeats, li
     subprocess.call(['git', 'commit', '-m', '\'Create red.yml file.\''])
     subprocess.call(['git', 'push'])
 
+    # Get last experiment ID
+    last_cc_id = get_last_cc_experimentid(keyring_service)
+
+
     # EXECUTE THE RED-YML
     if keyring_service is None:
-        p = 'faice exec cc_execution_file.red.yml'
+        p = 'faice exec cc_execution_file.red.yml --disable-retry'
     else:
-        p = 'faice exec --keyring-service ' + keyring_service + ' cc_execution_file.red.yml'
-    # TODO: IS A LIST NEEDED?
-    subprocess.call(p.split(' '))
+        p = 'faice exec --keyring-service ' + keyring_service + ' cc_execution_file.red.yml --disable-retry'
+    message = subprocess.call(p.split(' '))
     cc_id = get_last_cc_experimentid(keyring_service)
+    if last_cc_id == cc_id:
+        raise RuntimeError(bcolors.FAIL+'The job could not be started. Maybe there is some error in the '
+                                        'RED-YML file.\n\nMessage of "faice exec":\n' + message + bcolors.ENDC)
     print('The experiment ID is: ' + str(cc_id))
     return cc_id
 
@@ -594,14 +602,23 @@ def main():
             print(bcolors.BOLD + 'Create a DVC file for executing ' + ipynb_files_in_main_dir + ' with papermill.' +
                   bcolors.ENDC)
             from dvc_cc.run import papermill_helper
-            parameters = papermill_helper.read_params_from_parametercell(ipynb_files_in_main_dir)
+            parameters = papermill_helper.read_parameters_from_parametercell(ipynb_files_in_main_dir)
+            outputs = papermill_helper.read_definitions_from_parametercell(ipynb_files_in_main_dir, 'outputs')
+            metrics = papermill_helper.read_definitions_from_parametercell(ipynb_files_in_main_dir, 'metrics')
 
-            cmd = 'papermill ' + ipynb_files_in_main_dir + ' ' + ipynb_files_in_main_dir[:-6] + '_output.ipynb'
+            cmd = 'papermill ' + ipynb_files_in_main_dir + ' ' + ipynb_files_in_main_dir[:-6] + '_output.ipynb --log-output -k python'
             for p in parameters:
-                cmd = cmd +' -k python -p ' + p[0] + ' {{' + p[0] + ':' + p[1] + ':None}}'
+                cmd = cmd +' -p ' + p[0] + ' {{' + p[0] + ':' + p[1] + ':None}}'
 
-            subprocess.call(['dvc-cc', 'hyperopt', 'new', '-d', ipynb_files_in_main_dir, '-o',
-                             ipynb_files_in_main_dir[:-6] + '_output.ipynb', '-f','papermill.dvc',cmd])
+            dvc_cc_command = ['dvc-cc', 'hyperopt', 'new', '-d', ipynb_files_in_main_dir, '-o',
+                             ipynb_files_in_main_dir[:-6] + '_output.ipynb', '-f','papermill.dvc']
+            for o in outputs:
+                dvc_cc_command = dvc_cc_command + ['-o', o[0]]
+            for m in metrics:
+                dvc_cc_command = dvc_cc_command + ['-M', m[0]]
+            dvc_cc_command = dvc_cc_command + [cmd]
+
+            subprocess.call(dvc_cc_command)
             subprocess.call(['git', 'add', 'dvc/.hyperopt/papermill.hyperopt'])  # TODO: build quite mode!
             subprocess.call(['git', 'commit','-q','-m', 'Create a dvc file for papermill.'])
             subprocess.call(['git', 'push', '-q', '-u', 'origin', exp_name + ':' + exp_name])
@@ -697,13 +714,15 @@ def main():
         ##########################
         subprocess.call(['git', 'checkout', startbranch])
         try:
-            subprocess.call(['rm', '-R', '.dvc/lock'])
+            subprocess.call(['rm', '-R', '.dvc/lock'], stdout=subprocess.DEVNULL,
+                                      stderr=subprocess.DEVNULL)
             print('Information: delete ".dvc/lock" manually.')
         except:
             print('Information: ".dvc/lock" does not exist.')
 
         try:
-            subprocess.call(['rm', 'cc_execution_file.red.yml'])
+            subprocess.call(['rm', 'cc_execution_file.red.yml'], stdout=subprocess.DEVNULL,
+                                      stderr=subprocess.DEVNULL)
             print('Information: delete "cc_execution_file.red.yml')
         except:
             print('Information: "cc_execution_file.red.yml" does not exist.')
